# PL/pgSQL functions

## file analyseChJSON.sql
 
Function **analyseChJSON** gets an ID of the charging station and returns the JSON contents from table *de_chStations* as a table with the following structure (LevelID, PowerKW, Quantity, ConnectionTypeID, Amps, Voltage).

## file create\_insert\_waitingTypes.sql 

The table *WaitingTypes* is created with a structure: (ID, timeFrom, timeTo) where ID is a varchar notation. The record represents the type of waiting type with an interval thats starts with 0 and ends with a maximum waiting time. The table is filled with records, e.g. ('rapid', 0, 15) as rapid charging can take up to 15 minutes.


## file generateOneChargerProfile.sql 

Tha function **generateOneChargerProfile** generates a standard waiting profile, i.e. an interval of time for the provided charging station. The input is the an id of the charger. The output is an array of integers. The id must correspond to the ids in de\_chstations as function analyseChJSON is invoked.
Notice: currently minutes are generated as integers, but later into standardwaitingprofiles they are inserted as numeric. Do we need a floating point granularity?

## file create\_StandardChargerProfile.sql  

Table *StandardChargerProfile* is created with structure (ID, wTime1, wTime2) where ID is the charger ID from de\_chStations and wTimeX represents waiting time in minutes. The data for each charging station is generated using function generateOneChargerProfile.

## file create\_insert\_chargerAvailabilityPlan.sql
  
Table *Plans* is created  and two default plans are inserted. Structure of table Plans is (id, measure) where id is just the plan number and measure is one of the following ho, qu, hf - where ho means hour, qu - quater, and hf is half an hour.  
Table *ChargerAvailabilityPlan* is created to have diffirent load during the week day and time of the day. The structure is (planID, startTime, endTime, weekday, coef). For each plan we have specific periods (granularity is defined in table Plans) where the load is different than default, i.e. stored in StandarChargerProfile. If some periods have the default load they are not inserted into the table. Now two plans are created just for a regular Monday  (weekday=1). Currently, planID 1 has some increase from 6:00 to 19:00 with even higher loads 7:00-8:00 and 17:00-18:00 with the highest load 18:00-19:00. PlanID 2 simulates business premises. The load details are: the standard load is 4:00-6:00 and 20:00-22:00. The dicreased by half load is 22:00-23:00 and 3:00-4:00. The minimum load (very high availability) is 0:00-3:00 and 23:00-24:00. During 13:00-20:00 there is a little increase and from 6:00 to 13:00 there is an increase with a peak 7:00-9:00. An increase means a shift of the interval with an increasing minimu waiting time.

## file waitingTime.sql

Function **waitingTime** gets theree paremeters: charging station ID, a timestamp, and a charger plan ID. For business premises plan with ID 2 is prepared. The function returns an interval of waiting time, i.e. an array of integers containing two values. 

**Note that the function automatically extracts day of the week. Thus, the timestamp must be of any Monday.**

The example of the fucntion invocation and debug output:

```
select waitingTime (89860, '2020-02-17 02:10', 2);
NOTICE:  typical waiting time: 0-40
NOTICE:  checking time: 02:00:00
NOTICE:  coef based on time: -1 -0.5
NOTICE:  calc availability -0.91666666666666666667
 waitingtime 
-------------
 {0,3}
(1 row)
```
          
             
## file chargingTime.sql

Function **chargingTime** gets 6 parameters: chStation ID, current SoC (e.g. 20 means 20\%), batteryKW, required SoC, carLimit, currentTime. **Note: Currently, the last three parameters are unimplemented.** The function decides on the charging time based on the highest power available. If the charging is considered as fast, the maximum SoC is 80\%. The function returns an array of two values: state of charge and time in minutes. The example is provided below (134482 is the charging station ID, 20 is the current state of charge, 40 is the battery power; in the result: 80 means 80\% SoC and 35 is the charging time). 

```
select chargingTime(134482, 20, 40, -1, -1, '2020-02-20 12:12');
NOTICE:  max power: 50
 chargingtime 
--------------
 {80,35}
(1 row)
```

## file allWaitingTimes.sql

Function allWaitingTimes gets an ID of the charging station and returns a sequence of waiting intervals as a text. The result contains 48 values separated by a space. The whole procedure (steps in psql and copy from the server) how to export is described in file **exportWaitingTimes.sql**. The result is stored in file **waitingTimesByHour.csv** with the first value as station ID and then 48 values to present time intervals: separator is a space.