CREATE OR REPLACE FUNCTION waitingTime(chStation INTEGER, tDay TIMESTAMP, testPlan INTEGER)
RETURNS INTEGER[]

AS
$BODY$
DECLARE
  wTime NUMERIC[];
  wT1 numeric;
  wT2 NUMERIC;
  availability NUMERIC := 1;
  availability1 NUMERIC := 1;
  weekD INTEGER;
  checkTime TIME;
  wInterval NUMERIC :=0;
  measureType char(2);
  measureValue INTEGER := 0;
  intervalValue VARCHAR;
  shift NUMERIC := 0;
  negative BOOLEAN := FALSE;
  interMinute NUMERIC;
  minutes INTEGER := 0;
BEGIN
   --select the measure of the tested plan: hour, half an hour, or quarter
   SELECT measure, units, measureValue||' minutes'
   INTO measureType, measureValue, intervalValue
   FROM Plans
   WHERE id = testPlan;

   IF (measureType IS NULL) THEN
     RAISE EXCEPTION 'Nonexistent test availability planID --> %', testPlan
      USING HINT = 'Please check plan ID';
   END IF;   

   --select typical waiting time interval
   SELECT wTime1, wTime2
   INTO wT1, wT2
   FROM StandardChargerProfile
   WHERE id = chStation;

   IF (wT1 IS NULL) THEN
     RAISE EXCEPTION 'Nonexistent cahger ID --> %', chStation
      USING HINT = 'Please check charger ID';
   END IF;

   --RAISE NOTICE 'typical waiting time: %', wT1||'-'||wT2;

   select extract(dow from tDay)
   INTO weekD;

   --convert timestamp to the required granularity
   IF (measureType = 'ho') THEN
     checkTime := cast(date_part('hour', tDay)||':00' as time);
   ELSIF (measureType = 'hf') THEN
     IF (date_part('minute', tDay) <30) THEN
       checkTime := cast(date_part('hour', tDay)||':00' as time);
     ELSE
       checkTime := cast(date_part('hour', tDay)||':30' as time);
     END IF;
   ELSIF(measureType = 'qu') THEN
     IF (date_part('minute', tDay) <15) THEN
       checkTime := cast(date_part('hour', tDay)||':00' as time);
     ELSIF (date_part('minute', tDay) >=15 AND date_part('minute', tDay) < 30)
       THEN
       checkTime := cast(date_part('hour', tDay)||':15' as time);
     ELSIF (date_part('minute', tDay) >=30 AND date_part('minute', tDay) < 45)
       THEN
       checkTime := cast(date_part('hour', tDay)||':30' as time);
     ELSE
       checkTime := cast(date_part('hour', tDay)||':45' as time);
     END IF;
   
   END IF;
   --RAISE NOTICE 'checking time: %', checkTime;


   --check increase/decrease in usage of the charging station based on the time/day
   SELECT S.coef, E.coef
   INTO availability, availability1
   FROM ChargerAvailabilityPlan AS S FULL OUTER JOIN 
   (SELECT planID, CASE WHEN startTime='00:00' THEN '24:00'
                          ELSE startTime END, endTime, weekday, coef
    FROM ChargerAvailabilityPlan 
    WHERE planID = testPlan AND weekday = weekD
    )
    AS E ON (S.endTime = E.startTime)
   WHERE S.planID = testPlan AND S.startTime = checkTime AND S.weekday = weekD

   UNION

   SELECT NULL, E.coef
   FROM ChargerAvailabilityPlan as E 
   WHERE E.planID = testPlan  AND E.startTime = checkTime +intervalValue::interval AND E.weekday = weekD AND NOT EXISTS (SELECT * FROM ChargerAvailabilityPlan WHERE endTime = E.startTime)
;
   
   --RAISE NOTICE 'coef based on time: % %', availability, availability1;
   

   IF (availability IS NULL) THEN availability := 0; END IF;
   IF (availability1 IS NULL) THEN availability1 := 0; END IF;

   minutes := date_part('minute', tDay);

   IF ((availability <0 AND availability1 <0) OR (availability <0 AND availability1 =0) OR (availability =0 AND availability1 <0)) THEN
     BEGIN
       negative := TRUE;
       availability := availability *(-1);
       availability1 := availability1 *(-1);
     END;
   ELSIF ((availability <0 AND availability1 >0) OR (availability >0 AND availability1 <0)) THEN
     BEGIN
       interminute := (measureValue * abs(availability))/(abs(availability1)+abs(availability));
        --RAISE NOTICE 'interminute: %', interminute;
	IF (minutes < interminute) THEN
	  BEGIN
	    IF (availability < 0) THEN
	     negative := TRUE;
	     availability := availability *(-1);
	    END IF;
  	    measureValue := interminute;
	    availability1 := 0;
	  END;
	ELSE
	  BEGIN
	    IF (availability1 < 0) THEN
	     negative := TRUE;
	     availability1 := availability1 *(-1);
	    END IF;
	    availability := 0;
	    minutes := minutes - interminute;
	    measureValue := measureValue - interminute;
	  END;
	END IF;
     END;
   END IF;
     
     IF (availability < availability1) THEN
       BEGIN
         shift := availability;
         availability := minutes*(availability1-shift)/measureValue;
       END;
     ELSIF (availability > availability1) THEN  
       BEGIN     
         shift := availability1;
         availability := (measureValue - minutes)*(availability-shift)/measureValue;
       END;
     ELSE
       BEGIN
        shift := 0;
        
       END;
     END IF;
     
     availability := availability + shift;
     IF (negative) THEN availability := availability *(-1); END IF;
     --RAISE NOTICE 'calc availability %', availability;
     wInterval := (wT2 - wT1);
     wTime[1] := wT1 + wInterval * (availability);
     IF (wTime[1] < 0) THEN wTime[1] := 0; END IF;
     wTIme[2] := wT2 + wT2 * (availability);
     IF (wTime[2] < 0) THEN wTime[2] := 0; END IF;

   RETURN wTime;
END
$BODY$
LANGUAGE plpgsql;
