//=======================================================================================
// Project     : Charge-Arrival-Time Profiles
// Description : Charging stations
// ======================================================================================

#include "cst.h"

using namespace CATs;
using namespace std;
using namespace settings;

CATs::allWT CSt::allStationsWT = CATs::allWT();
CATs::allCHF CSt::allStationsCHF = CATs::allCHF();
CATs::MapKatch CSt::stMap = CATs::MapKatch();

//-----------------------------------------------------------------------------
// precondition: ts % delta = 0
void CSt::loadWTIfNeeded(Time ts) const
{
    if (cached_wt.find(ts) == cached_wt.end())
    {
        /*! load the wt interval corresponding to ts and possibly more
         *
         */
        //cached_wt[ts] = {0, 0};     // repace 0, 0 with the value loaded
         cached_wt = allStationsWT[id];
    }
}

//-----------------------------------------------------------------------------

bool CSt::loadAllWT() {
    bool successOnLoad = CATs::reading::loadStWT(allStationsWT);
    if (successOnLoad){
        successOnLoad = loadCHF();
        if (successOnLoad)
            successOnLoad = CATs::reading::loadStationMapKatch(stMap);
    }
    return successOnLoad;
}

Interval<Time> CSt::getWT(Time t) {
    Time granulated = settings::Settings::granularity* (t / settings::Settings::granularity);
    return cached_wt[granulated];
}

bool CSt::loadCHF()
{
    bool result = CATs::reading::loadStCHF(allStationsCHF);
    return true;
}

Time CSt::getChargingTime(Charge currSoC, Charge reqSoC){
    Time t = 0;
    Charge iniSoC = 0;
    if (currSoC >= reqSoC || currSoC < 0 || reqSoC < 0) return -3; //invalid input data
    else if (chf.size() == 0) return -1; //no charging info is available for the charging station
    else if (chf.at(chf.size() - 1).ch < reqSoC) return -2; //the fast charging limits the highest SoC possible
    else{ // chf.size() > 0 and valid input
        t = chargingTime(reqSoC) - chargingTime(currSoC);
    }
    return t;
}

Charge CSt::getChargeLevel(Charge currSoC, Time chTime){
    Charge reachCharge = 0;
    chfp iniSoC = {0,0};
    if (chTime < 0 || currSoC < 0) return -3; //invalid time value
    else if (chf.size() == 0) return -1; //no charging info is available for the charging station
    Time projectTime = chargingTime(currSoC) + chTime;
    if (chf.at(chf.size() - 1).t <= projectTime) return chf.at(chf.size() - 1).ch; //max level will be reached
    for(chfp &c: chf) {
        if (projectTime > iniSoC.t && projectTime <= c.t) {
            reachCharge = iniSoC.ch + (projectTime - iniSoC.t) * (c.ch - iniSoC.ch) / (c.t - iniSoC.t);
            break;
        } else
            iniSoC = c;
    }
    return reachCharge;
}

Time CSt::chargingTime(Charge SoC) { //from 0 to SoC
    Time t = 0;
    chfp iniSoC = {0,0};
    if (chf.at(chf.size() - 1).ch < SoC) return -2; //the fast charging limits the highest SoC possible
    for(chfp &c: chf){
        if (SoC > iniSoC.ch && SoC <= c.ch) {
            t = iniSoC.t + (c.t - iniSoC.t)*(SoC - iniSoC.ch)/(c.ch - iniSoC.ch);
            break;
        }
        else
            iniSoC = c;
    }
    return t;
}