//=======================================================================================
// Project     : Charge-Arrival-Time Profiles
// Description : Charging stations
// ======================================================================================

#ifndef CAT_PROFILES_CST_H
#define CAT_PROFILES_CST_H

#include "base.h"
#include <unordered_map>
#include <vector>
#include "reading.h"
#include "settings.h"

namespace CATs {

//-----------------------------------------------------------------------------
// An  instantiation of a charging station on a path to destination
//
class CSt
{
public:
    static allWT  allStationsWT;
    static allCHF allStationsCHF;
    static MapKatch stMap;

    CSt() {} ;
    CSt(WT&& twt, CHF&& tchf) : cached_wt{std::move(twt)}, chf{std::move(tchf)} {};  /// For testing

    CSt(int id, int KaTCHid, int distance) {this->id  = id; this->nodeID = KaTCHid; this->distance = distance; loadWTIfNeeded(0); chf = allStationsCHF[id];}

    Interval<Time> getWT(Time t);
    Time   getChargingTime(Charge currSoC, Charge reqSoC);
    Charge getChargeLevel(Charge currSoC, Time chTime);
    NodeID getNodeID() const { return nodeID; };

    //!
    //! \param granularity
    //! \return
    static bool loadAllWT();

    //!
    //! \return
    static bool loadCHF();

    Time    timeToD;      /// Pure drive time to destination

private:
    CStID    id {-1};   /// Charging station id
    NodeID   nodeID {-1};
    double distance;

    mutable WT  cached_wt;
            CHF chf;
    void loadWTIfNeeded(Time ts) const;    /// precondition: ts % delta = 0
    Time chargingTime(Charge SoC);
};

//-------------------------------------------------------------------------------
using CStSet = std::vector<CSt>;

}
#endif //CAT_PROFILES_CST_H
