//
// Created by agne on 2020-11-11.
//

#include "dbwrapper.h"

bool CATs::dbwrapper::connectToDB(){
    try {
        conn = make_unique<connection>(connString);
        if (conn->is_open()) {
            cout << "Connected to the database successfully: " << conn->dbname() << endl;
        } else {
            cout << "NOTICE: Unable to connect to the database. " << endl;
            return false;
        }
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
    return true;
}

std::vector<CATs::CSt> CATs::dbwrapper::getStationsAroundPath(const GraphRoute& path, int bufSize){
    vector<string> pairs = path;
    vector<CSt> stations;
    try {
        result resprep = execute_findStationsAround(bufSize, pairs);
        for (auto row: resprep) {
            CSt station(row[0].as<int>(), row[1].as<int>(), row[2].as<double>());
            stations.push_back(station);
        }
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
        return vector<CATs::CSt>();
    }
    return stations;
}

CATs::TEBounds CATs::dbwrapper::energy_consumption_ti(const GraphRoute& route, Interval<Time> ts)
{ // Call Alminas'  energy_on_path_minmax_bounds and fill rez ...
    TEBounds  rez{};
    try {
        row resprep = execute_energy_on_path_minmax_bounds(route, ts.b, ts.e);
        TE parts[2]{};
        int i = 0;
        for (auto col = std::begin(resprep); col != std::end(resprep); col++){
            Time t = static_cast<Time>(col->as<float>());
            auto c = (++col)->as<Charge>();
            auto s = (++col)->as<size_t>();
            auto str = (++col)->as<string>();

            cout << " masyviukas: "<< str <<endl;
            vector<Time> v = arr2TVec(str);
            /*for (Time t: v)
                cout<<"el:"<<t<<endl;*/

            parts[i++] = {t, c, s, v};
        }
        rez.first = parts[0]; rez.last = parts[1];
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
    return rez;
}

CATs::TEBoundsPlus CATs::dbwrapper::energy_consumption_tp(const GraphRoute& route, Time ts,
                                                          Charge max_energy,
                                                          double factor1, double factor2)
{ // Call Alminas'  energy_on_path_DT_interval and fill rez ...
    TEBoundsPlus rez{};

    try {
        row resprep = execute_energy_on_path_DT_interval(route, ts, max_energy, factor1, factor2);
        TE parts[4]{};
        int i = 0;
        for (auto col = std::begin(resprep); col != std::end(resprep); col++){
             parts[i++] = {static_cast<Time>(col->as<float>()), (++col)->as<Charge>(), (++col)->as<size_t>()};
        }
        rez.first = parts[0]; rez.last = parts[1]; rez.mid1 = parts[2]; rez.mid2 = parts[3];

    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
        //return vector<CATs::CSt>();
    }
    return rez;
}

void CATs::dbwrapper::prepare_findStationsAround(){
    conn->prepare(
           "stationsAround",
           "SELECT chid, exp_node_id, dist FROM station_around_path($2::text[], $1)");
}

pqxx::result CATs::dbwrapper::execute_findStationsAround(int bufSize, const vector<string>& vec){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared("stationsAround", bufSize, full);
}

void CATs::dbwrapper::prepare_energy_on_path_minmax_bounds(){ //nenaudojama kol kas
    conn->prepare(
            "energyMinMaxBounds",
            "SELECT * FROM energy_on_path_minmax_bounds($1::text[], $2, $3, $4)");
}

void CATs::dbwrapper::prepare_energy_on_path_minmax_bounds_noMax(){
    conn->prepare(
            "energyMinMaxBoundsNoMax",
            "SELECT s->'first'->>'time', s->'first'->>'energy', s->'first'->>'size', s->'first'->>'path_time',  "
            "s->'last'->>'time', s->'last'->>'energy', s->'last'->>'size', s->'last'->>'path_time'  "
            " FROM energy_on_path_minmax_bounds($1::text[], $2, $3) AS s");

}
row CATs::dbwrapper::execute_energy_on_path_minmax_bounds(const vector<string>& vec, Time s, Time e, double maxEnergy){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared1("energyMinMaxBounds", full, s, e, maxEnergy);
}

row CATs::dbwrapper::execute_energy_on_path_minmax_bounds(const vector<string>& vec, Time s, Time e){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared1("energyMinMaxBoundsNoMax", full, s, e);
}

void CATs::dbwrapper::prepare_energy_on_path_DT_interval(){
    char full[1000] =  "SELECT s->'first'->>'time', s->'first'->>'energy', s->'first'->>'size', "
                        "s->'last'->>'time', s->'last'->>'energy', s->'last'->>'size', "
                        "s->'mid1'->>'time', s->'mid1'->>'energy', s->'mid1'->>'size', "
                        "s->'mid2'->>'time', s->'mid2'->>'energy', s->'mid2'->>'size'"
                        "FROM energy_on_path_DT_interval($1::text[], $2, $3, $4, $5) AS s";
    conn->prepare(
            "energyDTinterval",
             full);
}
row CATs::dbwrapper::execute_energy_on_path_DT_interval(const vector<string>& vec, Time s, double maxEnergy, double factor1, double factor2){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared1("energyDTinterval", full, s, factor1, factor2, maxEnergy);
}

std::vector<CATs::StartEnd> CATs::dbwrapper::getTaskRoutes() {
    vector<StartEnd> v;
    try{
        nontransaction n(*conn);
        result res = n.exec("SELECT source, target FROM task_routes");
        for (auto const &row: res){
            CStID s = row[0].as<int>(), e = row[1].as<int>();;
            v.push_back({s,e});
        }
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
    return v;
}

std::vector<CATs::Time> CATs::dbwrapper::arr2TVec(std::string &str) {
    std::string clean = str.substr(1,str.size()-2);
    //cout << "išvalytas: "<<clean<<endl;
    size_t pos_start = 0, pos_end;
    string token;
    vector<string> res;
    vector<Time> cont;

    while ((pos_end = clean.find (",", pos_start)) != string::npos) {
        token = clean.substr (pos_start, pos_end - pos_start);
        pos_start = pos_end + 1;

        double val = roundl(std::stod(token)*10);
        cont.push_back(static_cast<Time>(val));  //deci-seconds
        //cout <<token <<endl;
        //cout <<val <<endl;
    }
    token = clean.substr (pos_start);
    double val = roundl(std::stod(token)*10);
    cont.push_back(static_cast<Time>(val));
    //cout <<token <<endl;
    //cout <<val <<endl;
    //cout<<"masyviuko ilgisL "<<cont.size()<<endl;

    return cont;
}

void CATs::dbwrapper::prepare_getTrips_calibrate_log() {
    conn->prepare(
            "getTrips",
            "SELECT id, source_katch, target_katch, time_katch "
            " FROM public.calibrate_log order by id ");
}

void CATs::dbwrapper::prepare_insertTrip_KATCH_calibrate_log() {
    conn->prepare(
            "updateKATCHTrip",
            "UPDATE public.calibrate_log SET path = $2::text[] WHERE id = $1 AND path IS NULL ");
}

result CATs::dbwrapper::execute_getTrips_calibrate_log() {
    nontransaction n(*conn);
    return n.exec_prepared("getTrips");
}

void CATs::dbwrapper::insertTrip_KATCH_calibrate_log(long id, const CATs::GraphRoute& path) {
    try {
        string full;
        stringVec2string(path, full);
        transaction n(*conn);
        n.exec_prepared0("updateKATCHTrip", id, full);
        n.commit();
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
}

vector<CATs::Trip> CATs::dbwrapper::getTrips_calibrate_log() {
    vector<Trip> v;
    try{
        result res = execute_getTrips_calibrate_log();
        for (auto const &r:res){
            v.push_back({r[0].as<long>(), r[1].as<NodeID>(), r[2].as<NodeID>(), r[3].as<Time>()});
        }
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
    return v;
}


