//
// Created by agne on 2020-11-11.
//

#ifndef CAT_PROFILES_DBWRAPPER_H
#define CAT_PROFILES_DBWRAPPER_H

#include <pqxx/pqxx>
#include "settings.h"
#include "cst.h"
#include <iostream>
#include <vector>
#include "stringTools.h"
#include <cmath>
//#include "base.h"

using namespace std;
using namespace pqxx;
using namespace settings;

namespace CATs{
    class dbwrapper{
    private:
        const std::string connString = "dbname = "+settings::Settings::nameDB+" user= "+settings::Settings::userDB+ " password = "+settings::Settings::passwdDB + " \
                             hostaddr = "+ settings::Settings::hostDB+ " port = "+settings::Settings::portDB;
        unique_ptr<connection> conn;
        void prepare_findStationsAround();
        result execute_findStationsAround(int bufSize, const vector<string>& vec);
        void prepare_energy_on_path_minmax_bounds();
        void prepare_energy_on_path_minmax_bounds_noMax();
        row execute_energy_on_path_minmax_bounds(const vector<string>& vec, Time s, Time e, double maxEnergy); //kol kas nenaudojama
        row execute_energy_on_path_minmax_bounds(const vector<string>& vec, Time s, Time e);
        void prepare_energy_on_path_DT_interval();
        row execute_energy_on_path_DT_interval(const vector<string>& vec, Time s, double maxEnergy, double factor1, double factor2);

        void prepare_getTrips_calibrate_log();
        result execute_getTrips_calibrate_log();
        void prepare_insertTrip_KATCH_calibrate_log();


        vector<Time> arr2TVec(std::string &str);
    public:
        dbwrapper(){
            cout << connectToDB() <<endl;
            prepare_findStationsAround();
            prepare_energy_on_path_minmax_bounds();
            prepare_energy_on_path_minmax_bounds_noMax();
            prepare_energy_on_path_DT_interval();
            prepare_getTrips_calibrate_log();
            prepare_insertTrip_KATCH_calibrate_log();
        }
        bool connectToDB();
        vector<CSt> getStationsAroundPath(const GraphRoute& path, int bufSize);
        void insertTrip_KATCH_calibrate_log(long id, const GraphRoute& path);
        vector<Trip> getTrips_calibrate_log();
        //-----------------------------------------------------------------------------
        /// Computes/retrieves travel time/energy consumption interval when leaving during a given time interval
        ///
        /// \param route
        /// \param ts Start time interval in seconds
        TEBounds     energy_consumption_ti(const CATs::GraphRoute& route, CATs::Interval<CATs::Time> ts);


        //-----------------------------------------------------------------------------
        /// Computes/retrieves travel time/energy consumption interval (and subintervals if factor1 !=0.0 and factor2 != 1.0)
        /// when leaving at a given time point
        ///
        /// \param route
        /// \param ts Start time in seconds
        /// \param factor1 subinterval bounds as fractions of the returned interval
        /// \param factor2
        /// \param max_energy stops computing if more energy than this is used
        TEBoundsPlus energy_consumption_tp(const CATs::GraphRoute& route, CATs::Time ts,
                                           CATs::Charge max_energy = std::numeric_limits<CATs::Charge>::max(),
                                           double factor1 = 0.0, double factor2 = 1.0);

        vector<StartEnd> getTaskRoutes();
    };
};

#endif //CAT_PROFILES_DBWRAPPER_H
