//=======================================================================================
// Project     : Charge-Arrival-Time Profiles
// Description : Main EV router algorithm and the RouteLeg class
// ======================================================================================

#include "ev_router.h"
#include "settings.h"
#include "katch.h"
#include <queue>

using namespace CATs;
using namespace std;
using namespace settings;
using namespace katch;

namespace CATs {

using CSTValue = double;       // Evaluation value (goodness) of a charging station

//-------------------------------------------------------------------------------
bool CSFilter(const CSt& cs, CSTValue best)
{
    return true;
}

//-------------------------------------------------------------------------------
CSTValue CSEvaluate(const CSt& cs)
{
    return 1;  // TODO: implement
}

//-------------------------------------------------------------------------------
CStSet ChooseCandidateCS(EVPath& path, Interval<Time> ts, Charge bc)
{
    auto max_energy = Settings::battery - bc;
    auto CSTset     = path.CStsAlong(max_energy, Settings::bufferCSt, ts);

    priority_queue<pair<CSTValue, const CSt*>>  candidates;

    for (const auto& cs: CSTset)
        if (CSFilter(cs, candidates.top().first))       // Consider the candidate CS
            candidates.push(make_pair(CSEvaluate(cs), &cs)); // Evaluate CS

    return CSTset;  // TODO: return only the candidates from the top of the priority queue
}
}

//-------------------------------------------------------------------------------
RouteSet CATs::Router(NodeID source, NodeID destination, Interval<Time> ts, Charge bc)
{
    auto max_energy = Settings::battery - bc;
    auto route      = EVPath{KaTCHRouting(source, destination, ts.midPoint())};
    auto CSTset     = ChooseCandidateCS(route, ts, bc);

    RouteSet result;

    if (CSTset.empty()) return result;  // not possible

    for (auto i = 0; i < Settings::results; i++);

    return result;
}

GraphRoute CATs::KaTCHRouting(NodeID source, NodeID destination, Time ts)
{
    TchEaQuery::Path  katchPath{0, 0};
    GraphRoute        rez{};

    TripRouter::getKaTCH().query(katchPath, static_cast<NodeIterator>(source), static_cast<NodeIterator>(destination),
                                 ts, 0, 4);
    for (auto i = 0; i < katchPath.get_n_edges(); i++)
    {
        auto [u, e, v] = katchPath.get_edge(i);
        rez.push_back(to_string(u)+"-"+to_string(v));
    }
    return rez;
}
