//=======================================================================================
// Project     : Charge-Arrival-Time Profiles
// Description : Main EV router algorithm and the RouteLeg class
// ======================================================================================

#ifndef CAT_PROFILES_EV_ROUTER_H
#define CAT_PROFILES_EV_ROUTER_H

#include "cat.h"
#include "cst.h"
#include <set>
#include <memory>
#include "base.h"

namespace CATs {

///  Route (represented by its last leg)

class RouteLeg
{
public:

    bool empty() const { return path.empty(); };

    bool operator<(const RouteLeg& other) const
    { return cat < other.cat; };

private:
    CSt                        cst;     /// Charging station at the beginning of the leg
    EVPath                     path;    /// The path of the leg
    CAT                        cat;     /// The CAT-profile of the route
    std::shared_ptr<RouteLeg>  parent;  /// pointer to the previous leg
};

using RouteSet = std::set<RouteLeg>;  /// The type for the answer set returned from the router

/// The main router procedure
/// Finds a set of routes from source to destination when starting during the interval time interval ts

RouteSet Router(NodeID source, NodeID destination, Interval<Time> ts, Charge bc);

//-----------------------------------------------------------------------------
/// Calls the CaTCH router and returns the edge list of the computed route
///
/// \param source
/// \param destination
/// \param ts Start time in seconds
    GraphRoute KaTCHRouting(NodeID source, NodeID destination, Time ts);
}



#endif //CAT_PROFILES_EV_ROUTER_H
