//
// Created by agne on 2020-11-09.
//

#ifndef DB_HANDLER_SETTINGS_H
#define DB_HANDLER_SETTINGS_H

#define MINI_CASE_SENSITIVE
#include "ini.h"
#include "base.h"

#define CAT_BE_VERBOSE true

#define CAT_STATUS(s) if (CAT_BE_VERBOSE) std::cout << "STATUS [" << __FILE__ << ":" << __LINE__  << "] " << s << std::flush
#define CAT_WARNING(s) if (CAT_BE_VERBOSE) std::cerr << "WARNING [" << __FILE__ << ":" << __LINE__  << "] " << s << std::flush
#define CAT_ERROR(s) if (CAT_BE_VERBOSE) std::cerr << "ERROR [" << __FILE__ << ":" << __LINE__  << "] " << s << std::flush

#define CAT_CONTINUE_STATUS(s) if (CAT_BE_VERBOSE) std::cout << s << std::flush
#define CAT_CONTINUE_ERROR(s) if (CAT_BE_VERBOSE) std::cerr << s << std::flush
#define CAT_CONTINUE_WARNING(s) if (CAT_BE_VERBOSE) std::cerr << s << std::flush



namespace settings{
    class Settings{
        public:
            Settings()= default;
            Settings(const std::string& ininame){
                 loadDefaults();
                 loadFromINI(ininame);
            };
            static void loadDefaults();
                   void loadFromINI(const std::string& );
            mINI::INIStructure globalINI;

            inline static CATs::Time  delta;  // Length of a time slot
            inline static int    granularity;
            inline static int    results ;    // number of results computed for each routing query
            inline static CATs::Charge battery;    // battery size in kWh
            inline static int    bufferCSt;  // buffer size around route in meters to look for stations
            inline static int    kCSt;    // # of Ch. Station candidates considered on each leg
            inline static double alpha;  // Alg. parameter in [0..1]. Small alpha = few ch. stops

            static std::string mappingStationsToKatch;// = "stationsMapKatch.csv";
            static std::string waitingTimesFile; //= "waitingTimesByHour.csv"; //Testing with file: waitingTimesTest.csv
            static std::string chargingFunctions; // = "chargingTimesBySoC_battery62.csv"; //Testing with file chargingFunctionsTest.csv
            static std::string userDB;
            static std::string passwdDB;
            static std::string nameDB;
            static std::string hostDB;
            static std::string portDB;
            static std::string btchfile;
    };
/*
    static mINI::INIStructure gINI;

    static int          granularity = 3600;  //granularity used in files of charging stations, 1 hour = 3600 sec; data presented for each hour in minutes
    static int          results     = 5;     // number of results computed for each routing query
    static CATs::Charge battery     = 75;    // battery size in kWh
    static int          bufferCSt   = 5000;  // buffer size around route in meters to look for stations
    static int          kCSt        = 10;    // # of Ch. Station candidates considered on each leg
    static double       alpha       = 0.1;   // Alg. parameter in [0..1]. Small alpha = few ch. stops

    static std::string mappingStationsToKatch;// = "stationsMapKatch.csv";
    static std::string waitingTimesFile; //= "waitingTimesByHour.csv"; //Testing with file: waitingTimesTest.csv
    static std::string chargingFunctions; // = "chargingTimesBySoC_battery62.csv"; //Testing with file chargingFunctionsTest.csv
    //const static int battery = 62;
    static std::string userDB;
    static std::string passwdDB;
    static std::string nameDB;
    static std::string hostDB;
    static std::string portDB;*/
}

#endif //DB_HANDLER_SETTINGS_H
