//
// Created by linb on 2021-01-07.
//

#ifndef CAT_PROFILES_TRIPROUTER_H
#define CAT_PROFILES_TRIPROUTER_H

#include "settings.h"
#include "katch.h"
#include "dbwrapper.h"

namespace CATs {

/// TRipRouter is a singlton class. Only one copy of it per thread is allowed at a time.
/// It is a collecting place for all the settings and interfaces to the external parts of the router
/// Some initialization happens in the constructor at the time of static initializations,
/// the rest can be done later through public methods.

// TODO: 1. Make the constructor private (requires a work-around for make_unique)
//       2. Remove the default construction for every thread - bad idea; instead require calling ReInit()

class TripRouter
{
public:
    TripRouter() : db{std::make_unique<dbwrapper>()} {};
    static void Init()
    { instance = std::make_unique<TripRouter>(); };

    static TripRouter& getIt()
    { return *instance; };

    /// \pre This method can be called only after KaTCH has been initialized with KaTCHinit()
    static KaTCH& getKaTCH()
    { return *instance->katch; };

    static dbwrapper& getDB()
    { return *instance->db; };

    int KaTCHinit();
    int SUMOcallibration(const int);
    int TestKaTCHSmall(const int);
private:
    inline static thread_local std::unique_ptr<TripRouter> instance{std::make_unique<TripRouter>()};

    std::unique_ptr<KaTCH>     katch;
    std::unique_ptr<dbwrapper> db;
    settings::Settings         defSets;
}; //class

};//namespace

#endif //CAT_PROFILES_TRIPROUTER_H
