//=======================================================================================
// Project     : Charge-Arrival-Time Profiles
// Description : Mock methods for CATCH2 unit testing
// ======================================================================================

#include "../dbwrapper.h"
#include "unit_tests.h"

bool CATs::dbwrapper::connectToDB()
{
    return true;
}

std::vector<CATs::CSt> CATs::dbwrapper::getStationsAroundPath(const GraphRoute& path, int bufSize){
    vector<string> pairs = path;
    vector<CSt> stations;
    try {
        result resprep = execute_findStationsAround(bufSize, pairs);
        for (auto row: resprep) {
            CSt station(row[0].as<int>(), row[1].as<int>(), row[2].as<double>());
            stations.push_back(station);
        }
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
        return vector<CATs::CSt>();
    }
    return stations;
}

CATs::TEBounds CATs::dbwrapper::energy_consumption_ti(const GraphRoute& route, Interval<Time> ts)
{
    TEBounds  rez{};
    try {
        row resprep = execute_energy_on_path_minmax_bounds(route, ts.b, ts.e);
        TE parts[2]{};
        int i = 0;
        for (auto col = std::begin(resprep); col != std::end(resprep); col++){
            Time t = static_cast<Time>(col->as<float>());
            auto c = (++col)->as<Charge>();
            auto s = (++col)->as<size_t>();
            auto str = (++col)->as<string>();

            cout << " masyviukas: "<< str <<endl;
            vector<Time> v = arr2TVec(str);
            /*for (Time t: v)
                cout<<"el:"<<t<<endl;*/

            parts[i++] = {t, c, s, v};
        }
        rez.first = parts[0]; rez.last = parts[1];
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
    return rez;
}

//-----------------------------------------------------------------------------------------------
// Mock method for testing
//
CATs::TEBoundsPlus CATs::dbwrapper::energy_consumption_tp(const GraphRoute& route, Time ts,
                                                          Charge max_energy,
                                                          double factor1, double factor2)
{
    TEBoundsPlus rez{};

    auto complTE    = [&max_energy](TE& te)
                      {
                           te.energy = UT::testCF(te.time);
                           te.size   = te.energy > max_energy ? 0 : 1;
                      };
    auto tti        = UT::testPath->tt(ts);
    auto ttl        = tti.len();

    rez.first.time   = tti.b;
    rez.last.time    = tti.e;
    complTE(rez.first);
    complTE(rez.last);

    rez.mid1.time = tti.b + static_cast<Time>(ttl * factor1);
    rez.mid2.time = tti.b + static_cast<Time>(ttl * factor2);
    complTE(rez.mid1);
    complTE(rez.mid2);

    return rez;
}

void CATs::dbwrapper::prepare_findStationsAround(){
}

pqxx::result CATs::dbwrapper::execute_findStationsAround(int bufSize, const vector<string>& vec){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared("stationsAround", bufSize, full);
}

void CATs::dbwrapper::prepare_energy_on_path_minmax_bounds(){ //nenaudojama kol kas
}

void CATs::dbwrapper::prepare_energy_on_path_minmax_bounds_noMax(){
}

row CATs::dbwrapper::execute_energy_on_path_minmax_bounds(const vector<string>& vec, Time s, Time e, double maxEnergy){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared1("energyMinMaxBounds", full, s, e, maxEnergy);
}

row CATs::dbwrapper::execute_energy_on_path_minmax_bounds(const vector<string>& vec, Time s, Time e){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared1("energyMinMaxBoundsNoMax", full, s, e);
}

void CATs::dbwrapper::prepare_energy_on_path_DT_interval(){
}

row CATs::dbwrapper::execute_energy_on_path_DT_interval(const vector<string>& vec, Time s, double maxEnergy, double factor1, double factor2){
    string full;
    stringVec2string(vec,full);
    nontransaction n(*conn);
    return n.exec_prepared1("energyDTinterval", full, s, factor1, factor2, maxEnergy);
}

std::vector<CATs::StartEnd> CATs::dbwrapper::getTaskRoutes() {
    vector<StartEnd> v;
    try{
        nontransaction n(*conn);
        result res = n.exec("SELECT source, target FROM task_routes");
        for (auto const &row: res){
            CStID s = row[0].as<int>(), e = row[1].as<int>();;
            v.push_back({s,e});
        }
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
    return v;
}

std::vector<CATs::Time> CATs::dbwrapper::arr2TVec(std::string &str) {
    std::string clean = str.substr(1,str.size()-2);
    //cout << "išvalytas: "<<clean<<endl;
    size_t pos_start = 0, pos_end;
    string token;
    vector<string> res;
    vector<Time> cont;

    while ((pos_end = clean.find (",", pos_start)) != string::npos) {
        token = clean.substr (pos_start, pos_end - pos_start);
        pos_start = pos_end + 1;

        double val = roundl(std::stod(token)*10);
        cont.push_back(static_cast<Time>(val));  //deci-seconds
        //cout <<token <<endl;
        //cout <<val <<endl;
    }
    token = clean.substr (pos_start);
    double val = roundl(std::stod(token)*10);
    cont.push_back(static_cast<Time>(val));
    //cout <<token <<endl;
    //cout <<val <<endl;
    //cout<<"masyviuko ilgisL "<<cont.size()<<endl;

    return cont;
}

void CATs::dbwrapper::prepare_getTrips_calibrate_log() {
}

void CATs::dbwrapper::prepare_insertTrip_KATCH_calibrate_log() {
}

result CATs::dbwrapper::execute_getTrips_calibrate_log() {
    nontransaction n(*conn);
    return n.exec_prepared("getTrips");
}

void CATs::dbwrapper::insertTrip_KATCH_calibrate_log(long id, const CATs::GraphRoute& path) {
    try {
        string full;
        stringVec2string(path, full);
        transaction n(*conn);
        n.exec_prepared0("updateKATCHTrip", id, full);
        n.commit();
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
}

vector<CATs::Trip> CATs::dbwrapper::getTrips_calibrate_log() {
    vector<Trip> v;
    try{
        result res = execute_getTrips_calibrate_log();
        for (auto const &r:res){
            v.push_back({r[0].as<long>(), r[1].as<NodeID>(), r[2].as<NodeID>(), r[3].as<Time>()});
        }
    }catch(const std::exception &e){
        cerr << e.what() << std::endl;
    }
    return v;
}


