//=======================================================================================
// Project     : Charge-Arrival-Time Profiles
// Description : Some CATCH2 unit tests of the main units
// ======================================================================================

#define CATCH_CONFIG_MAIN

#include "../catch.hpp"
#include "../ev_router.h"
#include "unit_tests.h"

using namespace CATs;
using namespace settings;

TEST_CASE("End-to-end CAT construction test", "[CAT]"){
    Settings::delta   = 100;
    Settings::battery = 1000;

    // Setting up the test data: Energy usage function and travel time function
    //
    UT::testCF = [](Time tt)->Charge
                 { return Settings::battery - (tt < 2*Settings::delta ?
                                               100 + (tt - 0.5*Settings::delta)*500/(1.5*Settings::delta) :
                                               600 + (tt - 2*Settings::delta)*200/(1.5*Settings::delta)); };
    EVPath::TT table { {0,       {{Settings::delta, 7*Settings::delta/2}, {}}},
                        {Settings::delta,   {{Settings::delta, 3*Settings::delta}, {}}},
                        {2*Settings::delta, {{3*Settings::delta/2, 3*Settings::delta}, {}}},
                        {3*Settings::delta, {{Settings::delta/2, 2*Settings::delta}, {}}} };
    for (auto& [dt, tten]: table)
    {
        tten.eci.b = UT::testCF(tten.tti.e);     // Note: For the above testCF, larger time means less energy
        tten.eci.e = UT::testCF(tten.tti.b);
    }
    EVPath  path{std::move(table)};
    UT::testPath = &path;
    std::ostringstream ostrA, ostrB, ostrC, ostrD, ostrE, ostrF;

    ostrA << CAT{path, {Settings::delta/2, Settings::delta/2}, Settings::battery};
    CHECK(ostrA.str() ==
          "[{{150,200},{266.667,433.333}}, {{200,300},{433.333,666.667}}, {{300,375},{666.667,766.667}}, \b\b]\n");
    ostrB << CAT{path, {Settings::delta/2, Settings::delta}, Settings::battery};
    CHECK(ostrB.str() ==
          "[{{150,200},{266.667,433.333}}, {{200,300},{266.667,666.667}}, {{300,400},{600,766.667}}, \b\b]\n");
    ostrC << CAT{path, {Settings::delta, 3*Settings::delta/2}, Settings::battery};
    CHECK(ostrC.str() ==
          "[{{200,300},{266.667,600}}, {{300,400},{433.333,733.333}}, {{400,450},{666.667,733.333}}, \b\b]\n");
    ostrD << CAT{path, {Settings::delta/2, 3*Settings::delta/2}, Settings::battery};
    CHECK(ostrD.str() ==
          "[{{150,200},{266.667,433.333}}, {{200,300},{266.667,666.667}}, {{300,400},{433.333,766.667}}, {{400,450},{666.667,733.333}}, \b\b]\n");
    ostrE << CAT{path, {Settings::delta, 2*Settings::delta}, Settings::battery};
    CHECK(ostrE.str() ==
          "[{{200,300},{266.667,600}}, {{300,400},{376.667,733.333}}, {{400,500},{600,733.333}}, \b\b]\n");
    ostrF << CAT{path, {2*Settings::delta, 3*Settings::delta}, Settings::battery};
    CHECK(ostrF.str() == "[{{350,400},{100,600}}, {{400,500},{266.667,733.333}}, \b\b]\n");

    CHECK(CAT{path, {Settings::delta, 2*Settings::delta}, Settings::battery/2}.isFeasible() == false);
}
