-- convert geometry to geography
ALTER TABLE public.de_chstations
    ADD COLUMN geogr geography;

UPDATE public.de_chstations SET geogr = geom::GEOGRAPHY;

CREATE INDEX "de_chstations_Geography"
    ON public.de_chstations USING gist
    (geogr)
    TABLESPACE pg_default;


ALTER TABLE public.grm_2po_4pgr
    ADD COLUMN geogr_way geography;

UPDATE public.grm_2po_4pgr SET geogr_way = geom_way::GEOGRAPHY;

CREATE INDEX "grm_2po_4pgr_Geography"
    ON public.grm_2po_4pgr USING gist
    (geogr_way)
    TABLESPACE pg_default;

  
-- This function will list charging stations within given distance from a given path
-- path is represented as an array of 'sourcetarget' from table (pgrout_export)
-- the results will be sorted by euclidian distance to the first segment in path
-- result is a table with chargins station_id, chargin station node_id on road network (use this for routing in KATCH), (ch_station geometry object for testing and distance from first edge)
 
CREATE OR REPLACE FUNCTION station_around_path(edges text[], dist integer)
                  RETURNS TABLE(chid integer, exp_node_id integer, ch_geo geography, dist float)  AS $$ 
    
	WITH buff as (SELECT distinct on (ch.id) ch.id, ch.snap_node_id, ch.geogr
              FROM (SELECT geogr_way from grm_2po_4pgr pg 
					                     JOIN 
					                     t_pgrout_export cpp ON (cpp.edge_id = pg.id) 
					                  where cpp.sourcetarget = ANY(edges)) as path
                     JOIN de_chstations ch ON ST_DWithin(geogr_way, geogr, dist) ),
     fst as  (SELECT geogr_way from grm_2po_4pgr pg 
			                        JOIN t_pgrout_export cpp ON 
			                          (cpp.edge_id = pg.id and cpp.sourcetarget = edges[1]))

     SELECT buff.id, cpp.id, buff.geogr, ST_Distance(geogr_way, geogr) as dist 
	   FROM buff 
	     JOIN t_pgrout_vertex_export cpp ON (cpp.node_id = buff.snap_node_id), fst 
       ORDER BY buff.geogr <-> fst.geogr_way;
$$
LANGUAGE SQL;
-- This function will list charging stations within given distance from a given path
-- path is represented as an array of 'sourcetarget' from table (pgrout_export)
-- the results will be sorted by euclidian distance to the first segment in path
-- result is a table with chargins station_id, chargin station node_id on road network (use this for routing in KATCH), (ch_station geometry object for testing and distance from first edge)
 

--test
select * from station_around_path('{"1-26731","26731-26735","26735-363051","363051-119619"}'::text[],100000);


--Successfully run. Total query runtime: 26 secs 897 msec.
--193 rows affected.


-- convinience function will extract chstations by given path_id which is a id in table path_log.

CREATE OR REPLACE FUNCTION station_around_path_id(path_id integer, dist integer)
                  RETURNS TABLE(chid integer, exp_node_id integer, ch_geo geography, dists float)  AS $$ 
DECLARE
   edges text[];
BEGIN
   select path into edges from path_log where id = path_id;
   return query
    select  * from station_around_path(edges, dist);
END;
$$
LANGUAGE plpgsql;
--TEST
--select * from station_around_path_id(1, 5000);

CREATE OR REPLACE FUNCTION station_around_path_nameid(path_name text, dist integer)
                  RETURNS TABLE(chid integer, exp_node_id integer, ch_geo geography, dists float)  AS $$ 
DECLARE
   edges text[];
BEGIN
   select path into edges from path_log where name = path_name;
   return query
    select  * from station_around_path(edges, dist);
END;
$$
LANGUAGE plpgsql;

--TEST
--select * from station_around_path_nameid('1-6', 5000);

