
-- tai funkcija skirta istraukti geometrija ta pacia tvarka kaip ir katch gautas kelias
-- INPUT:
-- edges - katch source-target id string array
-- OUTPUT: 
-- id - edge id from main network taable grm_2po_4pgr
-- osm_id - edge osm id
-- souce - egde source node on the main network
-- target - egte target node on the main network 
-- source_katch - egde source node using katch node id's
-- target_katch - edge target node using katch node id's
-- sourcetarget - edge id from katch
-- rownum - the path leg number in the same order as in given array
-- geogr_way - geography object for given road segment

CREATE OR REPLACE FUNCTION get_path_geometry(edges text[])
                  RETURNS TABLE(id integer, osm_id bigint, 
				                source integer, target integer,
								source_katch bigint, target_katch bigint,
								sourcetarget text, 
							    geogr_way geography, geom_way geometry, km float, ms float, rownum bigint)  AS $$ 
DECLARE
BEGIN
 create temp  sequence IF NOT EXISTS temp_seq;
 PERFORM setval('temp_seq', 1, false);
 
 return query
   select A.id, A.osm_id, A.source, A.target, B.source_katch, B.target_katch, B.sourcetarget, A.geogr_way, A.geom_way,  A.km, A.ms, B.ROWNUM
    from grm_2po_4pgr A,
        (select E.edge_id, E.source_katch, E.target_katch, E.sourcetarget, P.ROWNUM
          from t_pgrout_export AS E,
               (select unnest(edges) st, nextval('temp_seq') as ROWNUM ) AS P
          where E.sourcetarget = P.st ) AS B
    where A.id = B.edge_id
    order by B.ROWNUM;
END;
$$
LANGUAGE plpgsql;

-- convinience function will extract chstations by given path_id which is a id in table path_log.
CREATE OR REPLACE FUNCTION get_path_id_geometry(path_id integer)
                  RETURNS TABLE(id integer, osm_id bigint, 
				                source integer, target integer,
								source_katch bigint, target_katch bigint,
								sourcetarget text, 
							    geogr_way geography,geom_way geometry, km float, ms float, rownum bigint)  AS $$ 
DECLARE
   edges text[];
BEGIN
   select path into edges from path_log where path_log.id = path_id;
   return query
    select  * from get_path_geometry(edges);   
END; 
$$
LANGUAGE plpgsql;
-- TEST
-- select * from get_path_id_geometry(1);

CREATE OR REPLACE FUNCTION get_path_nameid_geometry(path_name text)
                  RETURNS TABLE(id integer, osm_id bigint, 
				                source integer, target integer,
								source_katch bigint, target_katch bigint,
								sourcetarget text, 
							    geogr_way geography,geom_way geometry, km float, ms float, rownum bigint)  AS $$ 
DECLARE
   edges text[];
BEGIN
   select path into edges from path_log where path_log.name = path_name;
   return query
    select  * from get_path_geometry(edges);   
END; 
$$
LANGUAGE plpgsql;
-- TEST
-- select * from get_path_nameid_geometry('1-6');



CREATE OR REPLACE FUNCTION insert_path_geometry(edges text[])
                  RETURNS name text  AS $$ 

--http://geojson.io/

SELECT jsonb_build_object(
    'type',     'FeatureCollection',
    'features', jsonb_agg(feature)
)
FROM (
  SELECT jsonb_build_object(
    'type',       'Feature',
    'id',         id,
    'geometry',   ST_AsGeoJSON(geogr_way)::jsonb,
    'encodedpolyline',  ST_AsEncodedPolyline(ST_LineMerge(geom_way)),       
    'properties', jsonb_build_object(  --to_jsonb(row) - 'gid' - 'geom'
                     'km',     km,
                     'osm_id', osm_id)
  ) AS feature
  FROM (SELECT * FROM get_path_id_geometry(1)) row) features;


update path_log set 
 encoded_path = ST_AsEncodedPolyline(ST_LineMerge( ( select ST_Union(geom_way) from get_path_id_geometry(id)) ));


$$
LANGUAGE plpgsql;

-- test
-- select * from get_path_geometry('{"1-26731","26731-26735","26735-363051","363051-119619"}'::text[])



-- input:
-- start_node - startinis taskas nuo kurio pradeti zigzaga
-- distance - atstumas tarp target ir source tasku
-- amount - kiek tasku poru surasti
-- output: 
-- lentele su: stulpeliais:
-- source, target - katch algoritmui skirti taskai marsrute
-- geom_source, geom_target - tasku kooridnates
-- geom_edge - tiese tarp target ir source 

CREATE OR REPLACE FUNCTION select_distant_points_connected(start_node bigint, distance integer, amount integer)
                  RETURNS TABLE(source integer, target integer,
                                geom_source geometry, geom_target geometry,				  
							    geom_edge geometry, km integer)  AS $$ 
DECLARE
 i           integer := 0;
 attempt     integer := 300;
 start_point integer := 1;
BEGIN
 
 source := start_node;
 select a.geom_vertex into geom_source from grm_2po_vertex a, t_pgrout_vertex_export b
          where a.id = b.node_id and b.id = start_node; 
 km := distance;
 
 WHILE i < amount
 LOOP
    attempt := 100;
    LOOP
      SELECT p.id, v.geom_vertex into target, geom_target
        FROM grm_2po_vertex v, t_pgrout_vertex_export p 
        WHERE v.id = p.node_id AND 
	          ST_DWithin(v.geom_vertex, geom_source, distance + 10, false)
              AND 
              NOT ST_DWithin(v.geom_vertex, geom_source, distance, false)
  	  OFFSET floor(random()*attempt) LIMIT 1;
	  
	  IF target is not null 
	    THEN exit;
	  ELSE
	    attempt = attempt / 2;
		if attempt < 1 then exit; END IF;
	  END IF;
	
	END LOOP;
	
	geom_edge := ST_MakeLine(geom_source, geom_target);
	
    return next;
	i := i + 1;
	source := target;
	geom_source := geom_target;
 END LOOP;
END;
$$
LANGUAGE plpgsql;


--test
select * from select_distant_points_connected(1,400000, 20); 

-- reikalingas pluginas: tsm_system_rows 
-- input:
-- distance - atstumas tarp target ir source tasku
-- amount - kiek tasku poru surasti
-- output: 
-- lentele su: stulpeliais:
-- source, target - katch algoritmui skirti taskai marsrute
-- geom_source, geom_target - tasku kooridnates
-- geom_edge - tiese tarp target ir source 
CREATE OR REPLACE FUNCTION select_distant_points(distance integer, amount integer)
                  RETURNS TABLE(source integer, target integer,
                                geom_source geometry, geom_target geometry,				  
							    geom_edge geometry, km integer)  AS $$ 
DECLARE
 i           integer := 1;
 attempt     integer := 300;
 point       record;
 amount_pct  float;
BEGIN


 km := distance;
 select 200*amount/count(*)::FLOAT into amount_pct from t_pgrout_vertex_export;
 
 FOR point IN (select geom_vertex, b.id as node_id
               from grm_2po_vertex a, 
                    t_pgrout_vertex_export b TABLESAMPLE BERNOULLI(amount_pct)
               where a.id = b.node_id)
 LOOP
	geom_source := point.geom_vertex;
	source		:= point.node_id;
	
    attempt := 100;
    LOOP
      SELECT p.id, v.geom_vertex into target, geom_target
        FROM grm_2po_vertex v, t_pgrout_vertex_export p 
        WHERE v.id = p.node_id AND 
	          ST_DWithin(v.geom_vertex, geom_source, distance + 10, false)
              AND 
              NOT ST_DWithin(v.geom_vertex, geom_source, distance, false)
  	   OFFSET floor(random()*attempt) LIMIT 1;
	  
	  IF target is not null 
	    THEN exit;
	  ELSE
	    attempt = attempt / 2;
		if attempt < 1 then exit; END IF;
	  END IF;
	END LOOP;
	
	geom_edge := ST_MakeLine(geom_source, geom_target);
    return next;
	i:= i+1;
	if i > amount THEN exit; END IF;	
 END LOOP;
END;
$$
LANGUAGE plpgsql;

-- test
select * from select_distant_points(300000, 20)
