create table pgrout_export(id serial primary key, edge_id bigint, source bigint, target bigint, points integer); --, ttf text);
\copy pgrout_export from '.\pgrout_export.csv'  csv header ;

CREATE TABLE public.de_chstations
(
  id integer,
  is_recently_verified boolean,
  date_last_verified date,
  uuid text,
  data_provider_id integer,
  operator_id integer,
  usagetype_id integer,
  addr_id integer,
  geom geometry(Point,4326),
  connections_js jsonb,
  connections_power_kw_total real,
  number_of_points integer,
  status_type_id integer,
  date_last_status_update date,
  data_quality_level integer,
  date_created date,
  submission_status_type_id integer,
  CONSTRAINT de_chstations_pkey PRIMARY KEY (id)		
)
TABLESPACE pg_default;

ALTER TABLE de_chstations 
   ADD COLUMN snap_node_id integer,
   ADD COLUMN snap_node_dist float;

DROP INDEX public.sidx_de_chstations_geom;

CREATE INDEX sidx_de_chstations_geom
    ON public.de_chstations USING gist
    (geom)
    TABLESPACE pg_default;
	
\copy de_chstations from '.\chstations.csv '  csv header ;

-- ----------------------------------------

drop table grm_2po_vertex;

CREATE TABLE grm_2po_vertex
(
    id integer NOT NULL primary key,
    clazz integer,
    osm_id bigint,
    osm_name character varying,
    ref_count integer,
    restrictions character varying,
    geom_vertex geometry(Point,4326)
);

CREATE INDEX idx_grm_2po_vertex_geom
    ON grm_2po_vertex USING gist
    (geom_vertex)
    TABLESPACE pg_default;

	
\copy grm_2po_vertexfrom '.\grm_2po_vertex.csv '  csv header ;


-- -----------------------------------


CREATE TABLE grm_2po_4pgr(id integer, osm_id bigint, osm_name character varying, osm_meta character varying, osm_source_id bigint, osm_target_id bigint, clazz integer, flags integer, source integer, target integer, km double precision, kmh integer, cost double precision, reverse_cost double precision, x1 double precision, y1 double precision, x2 double precision, y2 double precision);

SELECT AddGeometryColumn('grm_2po_4pgr', 'geom_way', 4326, 'MULTILINESTRING', 2);

ALTER TABLE grm_2po_4pgr 
   ADD COLUMN trfk_ms float,
   ADD COLUMN reverse_trfk_ms float,
   ADD COLUMN trfk_time float,
   ADD COLUMN reverse_trfk_time float;

ALTER TABLE grm_2po_4pgr 
   ADD COLUMN ms float,
   ADD COLUMN reverse_ms float,
   ADD COLUMN time float,
   ADD COLUMN reverse_time float;

ALTER TABLE grm_2po_4pgr 
   ADD COLUMN source_deg float,
   ADD COLUMN target_deg float,
   ADD COlUMN edge_radius float;

ALTER TABLE grm_2po_4pgr 
   ADD COLUMN source_h float,
   ADD COLUMN target_h float;

ALTER TABLE grm_2po_4pgr 
   ADD COLUMN loop_order int,
   ADD COLUMN loop_count int;

\copy grm_2po_4pgr from '.\grm_2po_4pgr.csv '  csv header ;

CREATE INDEX idx_grm_2po_4pgr_geom_way
    ON grm_2po_4pgr USING gist
    (geom_way)
    TABLESPACE pg_default;

--DROP INDEX public.idx_grm_2po_4pgr_source;
CREATE INDEX idx_grm_2po_4pgr_source
    ON grm_2po_4pgr USING btree
    (source ASC NULLS LAST)
    TABLESPACE pg_default;


-- DROP INDEX public.idx_grm_2po_4pgr_target;
CREATE INDEX idx_grm_2po_4pgr_target
    ON grm_2po_4pgr USING btree
    (target ASC NULLS LAST)
    TABLESPACE pg_default;



-- --------------------------

drop table pgrout_export;
create table pgrout_export(
  id serial primary key, 
  edge_id bigint, 
  source bigint,
  target bigint,
  points integer,
  km float,
  ms float,
  trfk_ms float,
  time float,
  trfk_time float,
  source_deg float,
  target_deg float,
  edge_radius float,
  source_h float,
  target_h float);


\copy pgrout_export from '.\pgrout_export.csv '  csv header ;

-- ------------------------------
DROP TABLE t_pgrout_export;

CREATE TABLE t_pgrout_export
(
    id serial primary key,
    edge_id bigint,
    source bigint,
    target bigint,
    source_katch bigint,
    target_katch bigint,
    sourcetarget text,
    points integer,
    km double precision,
    ms double precision,
    trfk_ms double precision,
    "time" double precision,
    trfk_time double precision,
    source_deg double precision,
    target_deg double precision,
    edge_radius double precision,
    source_h double precision,
    target_h double precision
);
\copy t_pgrout_export from '.\t_pgrout_export.csv '  csv header ;

CREATE INDEX targetsource
    ON t_pgrout_export USING hash
    (sourcetarget)
    TABLESPACE pg_default;

CREATE INDEX edgeid
    ON t_pgrout_export USING hash
    (edge_id)
    TABLESPACE pg_default;
-- ------------------------------
DROP TABLE t_pgrout_vertex_export;

CREATE TABLE t_pgrout_vertex_export
(
    id serial primary key,
    node_id bigint
);
\copy t_pgrout_vertex_export from '.\t_pgrout_vertex_export.csv '  csv header ;

DROP INDEX map_nodes;
CREATE INDEX map_nodes
    ON public.t_pgrout_vertex_export USING hash
    (node_id)
    TABLESPACE pg_default;




