drop table t_pgrout_export;
create table t_pgrout_export(
  id serial primary key, 
  edge_id bigint, 
  source bigint,
  target bigint,
  source_katch bigint,
  target_katch bigint,
  sourcetarget text,
  points integer,
  ttf text,
  km float,
  ms float,
  trfk_ms float,
  time float,
  trfk_time float,
  source_deg float,
  target_deg float,
  edge_radius float,
  source_h float,
  target_h float);

insert into t_pgrout_export (
 edge_id, source, target, points, ttf, km, ms, trfk_ms, time, trfk_time, source_deg, target_deg, edge_radius, source_h, target_h
 ) 
select 
  id,  
  source, 
  target,
--  3,
--  '0.0' ||' '|| COALESCE(round(time::numeric,1), 999999999) ||' '|| '432000' ||' '|| COALESCE(round((time * 0.5)::numeric,1),999999999) ||' '||  '468000.0' ||' '||  COALESCE(round(time::numeric,1),999999999)    
  CASE WHEN (round(LEAST(t_time, t_trfk_time)::numeric, 1) = round(GREATEST(t_trfk_time, t_time)::numeric, 1)) THEN 1 ELSE 96 END,
  CASE WHEN (round(LEAST(t_time, t_trfk_time)::numeric, 1) = round(GREATEST(t_trfk_time, t_time)::numeric, 1)) THEN '0 ' || round(LEAST(t_time, t_trfk_time)::numeric, 1) ELSE
    export_ttf_profile(LEAST(t_time, t_trfk_time), GREATEST(t_trfk_time, t_time)) END,
  km, 
  GREATEST(t_ms, t_trfk_ms), -- max - no traffic speed 
  LEAST(t_trfk_ms, t_ms),  -- min - traffic speed
  LEAST(t_time, t_trfk_time), -- min - no traffic time
  GREATEST(t_trfk_time, t_time), -- max - traffic time 
  source_deg,
  target_deg,
  edge_radius,
  source_h,
  target_h
from grm_2po_4pgr
where t_time > 0 and (loop_order is null or loop_order = 1)
order by id asc
--limit 30
;

insert into t_pgrout_export (
 edge_id, source, target, points, ttf, km, ms, trfk_ms, time, trfk_time, source_deg, target_deg, edge_radius, source_h, target_h
) 
select 
  id,  
  target,
  source, 
--  3,
--  '0.0' ||' '|| COALESCE(round(time::numeric,1), 999999999) ||' '|| '432000' ||' '|| COALESCE(round((time * 0.5)::numeric,1),999999999) ||' '||  '468000.0' ||' '||  COALESCE(round(time::numeric,1),999999999)    
  CASE WHEN (round(LEAST(t_reverse_time, t_reverse_trfk_time)::numeric, 1) = round(GREATEST(t_reverse_trfk_time, t_reverse_time)::numeric, 1)) THEN 1 ELSE 96 END,
  CASE WHEN (round(LEAST(t_reverse_time, t_reverse_trfk_time)::numeric, 1) = round(GREATEST(t_reverse_trfk_time, t_reverse_time)::numeric, 1)) THEN '0 ' || round(LEAST(t_reverse_time, t_reverse_trfk_time)::numeric, 1) ELSE
    export_ttf_profile(LEAST(t_reverse_time, t_reverse_trfk_time), GREATEST(t_reverse_trfk_time, t_reverse_time)) END,
  km, 
  GREATEST(t_reverse_ms, t_reverse_trfk_ms), -- max - no traffic speed 
  LEAST(t_reverse_trfk_ms, t_reverse_ms), -- min - traffic speed
  LEAST(t_reverse_time, t_reverse_trfk_time), -- min - no traffic time
  GREATEST(t_reverse_trfk_time, t_reverse_time), -- max - traffic time 
  CASE WHEN target_deg >= 180 THEN  target_deg - 180 ELSE target_deg + 180 END,
  CASE WHEN source_deg >= 180 THEN  source_deg - 180 ELSE source_deg + 180 END,
  edge_radius,
  target_h,
  source_h
from grm_2po_4pgr
where t_reverse_time > 0 and (loop_order is null or loop_order = 1)
order by id asc
--limit 30
;

delete from t_pgrout_export where source = target;

delete from t_pgrout_export where 
id in (select id from 
 (SELECT s.id,
    --s.osm_id,
    --s.geom_way,
    a.idn,
    a.cnt,
	ROW_NUMBER() OVER(PARTITION BY idn ORDER BY km asc) ord
   FROM  t_pgrout_export s -- grm_2po_4pgr s
     JOIN LATERAL ( SELECT (i.source || '-'::text) || i.target AS idn,
            i.source,
            i.target,
            count(*) AS cnt
           FROM t_pgrout_export i
          GROUP BY ((i.source || '-'::text) || i.target), i.source, i.target
		  HAVING count(*) > 1 ) a ON true
  WHERE a.cnt > 1 AND s.target = a.target AND s.source = a.source
  order by idn, km asc) b
where ord > 1 
order by idn, ord);

drop table t_pgrout_vertex_export;
create table t_pgrout_vertex_export(
  id serial primary key, 
  node_id bigint);

insert into t_pgrout_vertex_export (node_id)
select id from
(select distinct(source) as id from t_pgrout_export 
 UNION
select distinct(target) as id  from t_pgrout_export) a 
order by id asc;

DROP INDEX map_nodes;
CREATE INDEX map_nodes
    ON public.t_pgrout_vertex_export USING hash
    (node_id)
    TABLESPACE pg_default;

UPDATE t_pgrout_export 
SET source_katch = mapped.id -- ns
from (select n.source as ns, v.id as id from 
 t_pgrout_export n, t_pgrout_vertex_export v 
 where n.source = v.node_id) mapped where ns = source;

UPDATE t_pgrout_export 
SET target_katch = mapped.id --ns
from (select n.target as ns, v.id as id from 
 t_pgrout_export n, t_pgrout_vertex_export v 
 where n.target = v.node_id) mapped where ns = target;

UPDATE t_pgrout_export 
SET sourcetarget = source_katch||'-'||target_katch;


COPY 
--[UINT: number of nodes] [UINT: number of edges] [UINT: number of points] [UINT: period] [NEW LINE]</li>
(with 
   nodes as (select count(*)+1 cnt from t_pgrout_vertex_export), ----<li>Every node of the graph has a unique ID from the set {0,...,(number_of_nodes - 1)}.</li>
   edges as (select count(*) cnt from  t_pgrout_export),
   ttf as   (select sum(points) as cnt from  t_pgrout_export)
select nodes.cnt||' '||edges.cnt||' '||ttf.cnt||' '||'864000'  
from  nodes, edges, ttf 
UNION ALL
-->[UINT: source node id] [UINT: target node id] [UINT: number of points of this edges TTF]</li>
 select dump.str from (
  select edge_id, source_katch||' '||target_katch||' '||points||' '||ttf str from t_pgrout_export order by t_pgrout_export.edge_id asc) as dump )
 TO 'c:\DALTRA\db_handler\data\flow simulation\augment_netowkr_db\delta_import\export.tpgr';

ALTER TABLE t_pgrout_export DROP COLUMN ttf;
