drop table pgrout_export;
create table pgrout_export(
  id serial primary key, 
  edge_id bigint, 
  source bigint,
  target bigint,
  points integer,
  ttf text,
  km float,
  ms float,
  trfk_ms float,
  time float,
  trfk_time float,
  source_deg float,
  target_deg float,
  edge_radius float,
  source_h float,
  target_h float);
insert into pgrout_export (
 edge_id, source, target, points, ttf, km, ms, trfk_ms, time, trfk_time, source_deg, target_deg, edge_radius, source_h, target_h
 ) 
select 
  id,  
  source, 
  target,
--  3,
--  '0.0' ||' '|| COALESCE(round(time::numeric,1), 999999999) ||' '|| '432000' ||' '|| COALESCE(round((time * 0.5)::numeric,1),999999999) ||' '||  '468000.0' ||' '||  COALESCE(round(time::numeric,1),999999999)    
  24,
  export_ttf_profile(LEAST(time, trfk_time), GREATEST(trfk_time, time)),
  km, 
  GREATEST(ms, trfk_ms), -- max - no traffic speed 
  LEAST(trfk_ms, ms),  -- min - traffic speed
  LEAST(time, trfk_time), -- min - no traffic time
  GREATEST(trfk_time, time), -- max - traffic time 
  source_deg,
  target_deg,
  edge_radius,
  source_h,
  target_h
from grm_2po_4pgr
where time > 0
order by id asc
--limit 30
;

insert into pgrout_export (
 edge_id, source, target, points, ttf, km, ms, trfk_ms, time, trfk_time, source_deg, target_deg, edge_radius, source_h, target_h
) 
select 
  id,  
  target,
  source, 
--  3,
--  '0.0' ||' '|| COALESCE(round(time::numeric,1), 999999999) ||' '|| '432000' ||' '|| COALESCE(round((time * 0.5)::numeric,1),999999999) ||' '||  '468000.0' ||' '||  COALESCE(round(time::numeric,1),999999999)    
  24,
  export_ttf_profile(LEAST(reverse_time, reverse_trfk_time), GREATEST(reverse_trfk_time, reverse_time)),
  km, 
  GREATEST(reverse_ms, reverse_trfk_ms), -- max - no traffic speed 
  LEAST(reverse_trfk_ms, reverse_ms), -- min - traffic speed
  LEAST(reverse_time, reverse_trfk_time), -- min - no traffic time
  GREATEST(reverse_trfk_time, reverse_time), -- max - traffic time 
  CASE WHEN target_deg >= 180 THEN  target_deg - 180 ELSE target_deg + 180 END,
  CASE WHEN source_deg >= 180 THEN  source_deg - 180 ELSE source_deg + 180 END,
  edge_radius,
  target_h,
  source_h
from grm_2po_4pgr
where reverse_time > 0
order by id asc
--limit 30
;


COPY 
--[UINT: number of nodes] [UINT: number of edges] [UINT: number of points] [UINT: period] [NEW LINE]</li>
(with 
   nodes as (select count(*)+1 cnt from grm_2po_vertex), ----<li>Every node of the graph has a unique ID from the set {0,...,(number_of_nodes - 1)}.</li>
   edges as (select count(*) cnt from  pgrout_export),
   ttf as   (select sum(points) as cnt from  pgrout_export)
select nodes.cnt||' '||edges.cnt||' '||ttf.cnt||' '||'864000'  
from  nodes, edges, ttf 
UNION ALL
-->[UINT: source node id] [UINT: target node id] [UINT: number of points of this edges TTF]</li>
 select dump.str from (
  select edge_id, source||' '||target||' '||points||' '||ttf str from pgrout_export order by pgrout_export.edge_id asc) as dump )
 TO 'c:\DALTRA\data\flow simulation\augment_netowkr_db\export.tpgr';

ALTER TABLE pgrout_export DROP COLUMN ttf;
